/*
 *  SIPcrack | Martin J. Muench | mjm@codito.de | 2006
 *
 *  SIP password cracker
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <getopt.h>
#include <time.h>
#ifdef NO_OPENSSL
# include "md5.h"
#else
# include <openssl/md5.h>
#endif

#include "global.h"
#include "wrap.h"

#define VERSION     "0.1"

int opterr = 0; /* shutup getopt() */

static int  get_login_data(login_t *, const char *);
static int  crack_login_data(login_t *, const char *, char *, int, int);
static int  parse_sniffed_line(login_t *, char *);
static void usage(const char *);

int main(int argc, char *argv[])
{
  int c=0, ret, use_stdin=0, print_process=0;
  char *wordlist_file=NULL, *logins_file=NULL;
  char cracked_pass[PW_MAXLEN];
  login_t login_info;

  memset(&login_info, 0, sizeof(login_info));

  printf("\nSIPcrack %s  ( MaJoMu | www.remote-exploit.org ) \n"  \
         "--------------------------------------------------\n\n", VERSION);

  /* Get params */
  while((c = getopt(argc, argv, "d:w:p:s")) != -1) {
    switch(c) {
    case 'd':
      logins_file = (char *)Malloc(strlen(optarg)+1);
      strcpy(logins_file, optarg);
      break;
    case 'w':
      wordlist_file = (char *)Malloc(strlen(optarg)+1);
      strcpy(wordlist_file, optarg);
      break;
    case 's':
      use_stdin = 1;
      break;
    case 'p':
      print_process = atoi(optarg);
      break;
    default:
      usage("Invalid arguments");
    }
  }
  if((wordlist_file == NULL && !use_stdin) || logins_file == NULL)
    usage("Invalid arguments");

  /* Get Login data that shall be cracked */
  printf("* Reading and parsing dump file...\n");
  ret = get_login_data(&login_info, logins_file);
  if(ret < 0) {
    fprintf(stderr, "get_login_data() failed, exiting...\n");
    exit(EXIT_FAILURE);
  }

  /* Start wordlist brute force */
  ret = crack_login_data(&login_info, wordlist_file, cracked_pass, use_stdin, print_process);

  /* password cracked, update dump file */
  if(ret == 1) {
    printf("* Updating '%s'...", logins_file);
    fflush(stdout);
    update_login_data(&login_info, cracked_pass, logins_file);
    printf("done\n");
  }
  
  /* Clean up and exit */
  free(wordlist_file);
  free(logins_file);
  exit(EXIT_SUCCESS);
}

/* display content of dump file, cleanup coming soon */
static int get_login_data(login_t *login_info, const char *logins_file)
{
  FILE *lfile=NULL;
  char buffer[LOGIN_MAXLEN], input[12];
  char *login_buffer=NULL;
  login_t *login_ptr;
  unsigned int i=0, j=0, use=0;

  login_buffer = Malloc(sizeof(login_t));
  login_ptr = (login_t*)login_buffer;

  /* Try to open logins_file and read all logins to login_t structures */
  if((lfile = fopen(logins_file, "r")) == NULL) {
    fprintf(stderr, "* Cannot open dump file: %s\n", strerror(errno));
    free(login_buffer);
    return -1;
  }

  while((fgets(buffer, sizeof(buffer), lfile)) != NULL) {
      
      /* Copy buffer to struct, if fails ignore line */
      if(parse_sniffed_line(login_ptr, buffer) < 0)
	  continue;
      
      i++;
      
      login_buffer = Realloc(login_buffer, sizeof(login_t) * (i + 1));
      login_ptr = (login_t*)(login_buffer + sizeof(login_t) * i);
      
  }
  fclose(lfile);
  
  if(!i) {
      printf("No sniffed logins found, exiting\n");
      free(login_buffer);
      return -1;
  }
  
  /* print sniffed connections */
  printf("* Found Accounts:\n\nNum\tServer\t\tClient\t\tUser\tAlgorithm\tHash / Password\n\n");
  for(j = 0 ; j < i ; j++) {
    login_ptr = (login_t*)(login_buffer + sizeof(login_t) * j);

    printf("%d\t%s\t%s\t%s\t%s\t\t%s\n", 
	   j + 1,
	   login_ptr->server,
	   login_ptr->client,
           login_ptr->user,
           login_ptr->algorithm,
	   login_ptr->hash
	   );
    login_ptr = (login_t*)(login_buffer + sizeof(login_t) * (j + 1));
  }
  printf("\n");    

  /* Ask user which one to use */
  do {
    get_string_input(input, sizeof(input), "* Select which entry to crack (1 - %d): ", i);
    use = atoi(input);
  } while(!use || use > i);

  /* Copy structure to pointer */
  login_ptr = (login_t*)(login_buffer + sizeof(login_t) * (use - 1));

  if(!strncmp(login_ptr->algorithm, "PLAIN", sizeof(login_ptr->algorithm))) {
    printf("* Password already cracked: '%s'\n", login_ptr->hash);
    exit(EXIT_SUCCESS);
  }

  memcpy(login_info, login_ptr , sizeof(login_t));

  free(login_buffer);
  return 1;
}


static int crack_login_data(login_t *login, const char *wordlist, char *cracked_pw, int use_stdin, int print_process) 
{
  char static_hash[MD5_LEN_HEX+1], dynamic_hash[MD5_LEN_HEX+1], final_hash[MD5_LEN_HEX+1], password[PW_MAXLEN];
  char dynamic_hash_data[HOST_MAXLEN + USER_MAXLEN + 3], static_hash_data[NONCE_MAXLEN + MD5_LEN_HEX + 3];
  char bin2hex_table[256][2];
  FILE *passdb=NULL;
  MD5_CTX state;
  unsigned char md5_bin_hash[MD5_LEN];
  int found=0;
  unsigned int num_pass=0;
  time_t begin, end;
  size_t static_hash_data_len, dynamic_hash_data_len, pw_len=0;

  if(strncmp(login->algorithm, "MD5", strlen(login->algorithm))) {
    printf("\n* Only MD5 supported so far...\n");
    return -1;
  }

  printf("\n* Generating static %s hash...", login->algorithm);
  fflush(stdout);

  /* Init HEX table */
  init_bin2hex(bin2hex_table);

  /* Generating MD5 static hash */
  MD5_Init(&state);
  MD5_Update(&state, (unsigned char*)login->method, strlen( login->method ));
  MD5_Update(&state, (unsigned char*)":", 1);
  MD5_Update(&state, (unsigned char*)login->uri, strlen( login->uri ));
  MD5_Final(md5_bin_hash, &state);
  bin_to_hex(bin2hex_table, md5_bin_hash, MD5_LEN, static_hash, MD5_LEN_HEX);
  printf("%s\n", static_hash);

  /* Constructing first part of dynamic hash to save function calls */
  snprintf(dynamic_hash_data, sizeof(dynamic_hash_data), "%s:%s:", login->user, login->realm);

  /* Construct last part of final hash data */
  snprintf(static_hash_data, sizeof(static_hash_data), ":%s:%s", login->nonce, static_hash);

  /* Get lens of static buffers */
  dynamic_hash_data_len = strlen(dynamic_hash_data);
  static_hash_data_len  = strlen(static_hash_data);

  printf("* Starting bruteforce against user '%s' (%s Hash: '%s')\n",
	 login->user, login->algorithm, login->hash);

  if(!use_stdin) {
    /* Open wordlist file and try all words */
    if((passdb=fopen(wordlist, "rb")) == NULL) {
      fprintf(stderr, "* Cannot open wordlist '%s'\n", wordlist);
      return -1;
    }
    printf("* Loaded wordlist: '%s'\n", wordlist);
  }
  else {
    passdb=stdin;
    printf("* Type your passwords:\n");
  }

  time(&begin);

  /* Generate hashes and check for match */
  while(fgets(password, sizeof(password), passdb) != NULL) {

  num_pass++;

  /* generate dynamic hash */
  MD5_Init(&state);
  MD5_Update(&state, (unsigned char*)dynamic_hash_data, dynamic_hash_data_len);
  pw_len = strlen(password);
  MD5_Update(&state, (unsigned char*)password, (password[pw_len-2] == 0x0d ? pw_len-2 : password[pw_len-1] == 0x0a ? pw_len -1 : pw_len));
  MD5_Final(md5_bin_hash, &state);
  bin_to_hex(bin2hex_table, md5_bin_hash, MD5_LEN, dynamic_hash, MD5_LEN_HEX);

  /* generate response= hash and check */
  MD5_Init(&state);
  MD5_Update(&state, (unsigned char*)dynamic_hash, MD5_LEN_HEX);
  MD5_Update(&state, (unsigned char*)static_hash_data, static_hash_data_len);
  MD5_Final(md5_bin_hash, &state);
  bin_to_hex(bin2hex_table, md5_bin_hash, MD5_LEN, final_hash, MD5_LEN_HEX);

  if(!strncmp(final_hash, login->hash, MD5_LEN_HEX)) {
    found = 1;
    break;
  }

  if(print_process) {
    if(num_pass % print_process == 0)
      printf("* Already tried %d passwords (last '%s')\n", num_pass, password); 
  }

  } /* while */
  time(&end);

  if(!use_stdin)
    fclose(passdb);

  printf("* Tried %u passwords in %d seconds\n", num_pass, (int)(end - begin));

  if(found == 1) {
    /* remove %0d%0a */
    if(password[pw_len-2] == 0x0d)
      password[pw_len-2] = 0x00;
    else if(password[pw_len-1] == 0x0a)
      password[pw_len-1] = 0x00;

    printf("\n* Found password: '%s'\n", password);

    memset(cracked_pw, 0, PW_MAXLEN);
    strncpy(cracked_pw, password, PW_MAXLEN - 1);
    return 1;
  }
  else {
    printf("\n* Tried all passwords, no match\n");
    return 0;
  }
}

/* Parse sniffed login data line and copy to struct */
static int parse_sniffed_line(login_t *login, char *buffer) 
{
  char **lines;
  int num_lines, i;
  size_t len;

  memset(login, 0, sizeof(login_t));

  len = strlen(buffer);
  
  for(i = 0; i < len; i++) {
    if(buffer[i] == 0x0a) { buffer[i] = 0x00; break; }
    else if(buffer[i] == 0x0d) { buffer[i] = 0x00; break; }
  }

  lines = stringtoarray(buffer, '"', &num_lines);

  if(num_lines == 9) {
    strncpy(login->server,   lines[0], sizeof(login->server)    - 1 );
    strncpy(login->client,   lines[1], sizeof(login->client)    - 1 );
    strncpy(login->user,     lines[2], sizeof(login->user)      - 1 );
    strncpy(login->realm,    lines[3], sizeof(login->realm)     - 1 );
    strncpy(login->method,   lines[4], sizeof(login->method)    - 1 );
    strncpy(login->uri,      lines[5], sizeof(login->uri)       - 1 );
    strncpy(login->nonce,    lines[6], sizeof(login->nonce)     - 1 );
    strncpy(login->algorithm,lines[7], sizeof(login->algorithm) - 1 );
    strncpy(login->hash,     lines[8], sizeof(login->hash)      - 1 );
  }

  for(i = 0 ; i < num_lines ; i++) {
    free(lines[i]);
  }

  if(num_lines != 9)
    return -1;

  return 0;
}

/* show usage */
static void usage(const char *err_msg)
{
  printf("Usage: SIPcrack [ OPTIONS ] [ -s | -w <wordlist> ] -d <dump file>\n\n"    \
         "       -d dump file  = file containing logins sniffed by SIPdump \n" \
	 "       -w wordlist   = file containing all passwords to try      \n" \
	 "       Options:                                                  \n" \
         "       -s            = use stdin for passwords                   \n" \
	 "       -p num        = print cracking process every n passwords  \n" \
	 "                       (ATTENTION: slows down heavily)           \n" \
         "\n* %s\n", err_msg);
  exit(EXIT_FAILURE);
}
